import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.Shape;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.Ellipse2D;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;

public class LevelSelectPanel extends JPanel implements ActionListener
{
    public static final int N_LEVELS = 10;
    public static final int BTN_LEVEL_HEIGHT = 80;
    private JButton[] btnLevels = new JButton[N_LEVELS];
    private PlayerProfile profile;
    private ImageIcon bannerImage = new ImageIcon("LevelSelect.png");

    public LevelSelectPanel(PlayerProfile profile, String message)
    {
        this.profile = profile;

        setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

        JPanel pnlLevelsTop = new JPanel() {
            @Override
            protected void paintComponent(Graphics g)
            {
                super.paintComponent(g);
                g.setColor(Color.BLACK);
                g.fillRect(50, BTN_LEVEL_HEIGHT / 2, getWidth() - 100, 10);
                g.fillRect(512 + BTN_LEVEL_HEIGHT / 2, 50, 10, getHeight() - 50);
            }
        };

        JPanel pnlLevelsBot = new JPanel() {
            @Override
            protected void paintComponent(Graphics g)
            {
                super.paintComponent(g);
                g.setColor(Color.BLACK);
                g.fillRect(50, BTN_LEVEL_HEIGHT / 2, getWidth() - 100, 10);
                g.fillRect(512 + BTN_LEVEL_HEIGHT / 2, 0, 10, 50);
            }
        };

        pnlLevelsTop.setLayout(new FlowLayout());
        pnlLevelsBot.setLayout(new FlowLayout());
        
        pnlLevelsTop.setPreferredSize(new Dimension(getWidth(), BTN_LEVEL_HEIGHT));
        pnlLevelsBot.setPreferredSize(new Dimension(getWidth(), BTN_LEVEL_HEIGHT));

        for(int i = 0; i < N_LEVELS; i++)
        {
            btnLevels[i] = new JButton()
            {
                {
                    setPreferredSize(new Dimension(BTN_LEVEL_HEIGHT, BTN_LEVEL_HEIGHT));
                    setContentAreaFilled(false);
                }

                @Override
                protected void paintComponent(Graphics g)
                {
                    ((Graphics2D)g).setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
                    if(getModel().isArmed())
                    {
                        g.setColor(Color.lightGray);
                    }
                    else
                    {
                        g.setColor(getBackground());
                    }
                    g.fillOval(0, 0, (int)getSize().getWidth() - 1, (int)getSize().getHeight() - 1);
                    super.paintComponent(g);
                }

                @Override
                protected void paintBorder(Graphics g)
                {
                    ((Graphics2D)g).setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
                    g.setColor(getForeground());
                    g.drawOval(0, 0, getSize().width - 1, getSize().height - 1);
                }

                private Shape bounds;

                @Override
                public boolean contains(int x, int y)
                {
                    if(bounds == null || !bounds.getBounds().equals(getBounds()))
                    {
                        bounds = new Ellipse2D.Float(0, 0, getWidth(), getHeight());
                    }
                    return bounds.contains(x, y);
                }
            };
            
            btnLevels[i].addActionListener(this);
            
            int actualLevel;
                        
            if(i < N_LEVELS / 2)
            {
                actualLevel = i + 1;
                pnlLevelsTop.add(btnLevels[i]);
                if(i != N_LEVELS / 2 - 1)
                {
                    pnlLevelsTop.add(Box.createHorizontalStrut(30));
                }
            }
            else
            {
                actualLevel = N_LEVELS - i + 5;
                pnlLevelsBot.add(btnLevels[i]);
                if(i != N_LEVELS - 1)
                {
                    pnlLevelsBot.add(Box.createHorizontalStrut(30));
                }
            }

            btnLevels[i].setText("Stage " + actualLevel);
            btnLevels[i].setFocusable(false);
            btnLevels[i].setFocusPainted(false);
            
            if(actualLevel < profile.getLevel())
            {
                btnLevels[i].setBackground(new Color(32, 255, 32));
            }
            else if(actualLevel > profile.getLevel())
            {
                btnLevels[i].setBackground(new Color(95, 95, 95));
            }
            else
            {
                btnLevels[i].setBackground(new Color(255, 255, 32));
            }
        }

        JPanel pnlBanner = new JPanel() {
            @Override
            protected void paintComponent(Graphics g)
            {
                g.drawImage(bannerImage.getImage(), 0, 0, null);
            }
        };
        
        pnlBanner.setPreferredSize(new Dimension(getWidth(), 100));
        
        add(pnlBanner);

        JPanel pnlExit= new JPanel();
        pnlExit.setLayout(new BoxLayout(pnlExit, BoxLayout.Y_AXIS));
        pnlExit.setPreferredSize(new Dimension(getWidth(), 30));

        JButton btnExit = new JButton("Back to Menu");
        btnExit.addActionListener(this);
        btnExit.setFont(SlimeCollection.largeArial);

        JPanel pnlExitCenter = new JPanel();
        pnlExitCenter.add(btnExit);
        
        btnExit.setPreferredSize(new Dimension(200, btnExit.getPreferredSize().height));

        JLabel lblMessage = new JLabel(message, JLabel.CENTER);
        lblMessage.setFont(SlimeCollection.largeArial);
        
        pnlExit.add(Box.createVerticalGlue());
        pnlExit.add(lblMessage);
        pnlExit.add(Box.createVerticalGlue());
        pnlExit.add(pnlExitCenter);
        pnlExit.add(Box.createVerticalGlue());
        
        JPanel pnlShop = new JPanel();
        pnlShop.setLayout(new BoxLayout(pnlShop, BoxLayout.Y_AXIS));
        pnlShop.setPreferredSize(new Dimension(getWidth(), 30));
                
        JButton btnShop = new JButton("Shop");
        btnShop.addActionListener(this);
        btnShop.setFont(SlimeCollection.largeArial);
        
        JPanel pnlShopCenter = new JPanel();
        pnlShopCenter.add(btnShop);
        
        btnShop.setPreferredSize(new Dimension(200, btnShop.getPreferredSize().height));
        
        pnlShop.add(Box.createVerticalGlue());
        pnlShop.add(pnlShopCenter);
        pnlShop.add(Box.createVerticalGlue());
        
        add(pnlExit);
        add(pnlShop);
        add(pnlLevelsTop);
        add(pnlLevelsBot);
    }

    public void addNotify()
    {
        super.addNotify();
        if(profile.getActiveLevel() != 0)
        {
            SlimeCollection.switchFrames(this, new GamePanel(profile.getActiveLevel(), profile));
        }
    }
    
    public void actionPerformed(ActionEvent e)
    {
        String btnText = ((JButton)e.getSource()).getText();
        if(btnText.equals("Back to Menu"))
        {
            SlimeCollection.switchFrames(this, new MenuPanel());
        }
        else if(btnText.equals("Shop"))
        {
            SlimeCollection.switchFrames(this, new ShopPanel(profile));
        }
        else
        {
            try
            {
                String strLevel = btnText.split(" ")[1];
                int level = Integer.parseInt(strLevel);
                System.out.println("select " + level);
                if(level <= profile.getLevel())
                {
                    SlimeCollection.switchFrames(this, new GamePanel(level, profile));
                }
            }
            catch(NumberFormatException nfe)
            {
                nfe.printStackTrace();
            }
        }
    }
}
